(function ($) {
	'use strict';

	var HappyFormsSelect = function( el, options ) {
		this.el = el;
		this.$el = $( this.el );

		this.$input = options.$input;
		this.$select = options.$select;
		this.$placeholder = $( 'li.happyforms-custom-select-dropdown__placeholder', this.$select );
		this.searchable = options.searchable;

		this.defaultValue = '';
		this.defaultLabel = '';
		this.placeholderText = this.$placeholder.text();

		this.init();
	}

	HappyFormsSelect.prototype.init = function () {
		this.$el.attr('readonly', 'readonly');

		if ( this.searchable && 'false' !== this.searchable ) {
			this.$el.on( 'keyup', this.searchOptions.bind( this ) );
			this.$el.removeAttr( 'readonly' );
		}

		this.defaultValue = this.$input.val();
		this.defaultLabel = this.$el.val();

		if ( '' === this.defaultLabel && '' === this.defaultValue && this.$placeholder.length ) {
			this.defaultLabel = this.placeholderText;
		}

		this.$el.on( 'click', this.toggleDropdown.bind( this ) );
		this.$el.parent().on( 'click', this.toggleDropdown.bind( this ) );
		this.$select.on( 'click', 'li', this.onItemSelect.bind( this ) );
		$( window ).on( 'click', this.onOutsideClick.bind(this) );

		this.setValue();
	}

	HappyFormsSelect.prototype.toggleDropdown = function( e ) {
		e.stopPropagation();
		
		if ( ! this.$select.is( ':visible' ) ) {
			$( '.happyforms-custom-select-dropdown' ).hide();
			this.$select.show();
		} else {
			this.hideDropdown();
		}
	}

	HappyFormsSelect.prototype.onOutsideClick = function( e ) {
		if ( ! this.el.contains(e.target).length && this.$select.is( ':visible' ) ) {
			this.hideDropdown();
		}
	}

	HappyFormsSelect.prototype.hideDropdown = function() {
		this.$select.hide();
		$( 'li.happyforms-dropdown-item', this.$select ).show();
	}

	HappyFormsSelect.prototype.onItemSelect = function( e ) {
		e.stopPropagation();

		// if enter or tab
		if ( 'keyup' === e.type && 13 !== e.keyCode || 9 === e.keyCode ) {
			return false;
		}

		var $li = $( e.target );
		this.setValue( $li.data('value').toString() );
	}

	HappyFormsSelect.prototype.searchOptions = function ( e ) {
		// if tab
		if ( e.key && 9 === e.keyCode ) {
			return false;
		}

		var value = this.$el.val();
		var $allItems = $( 'li.happyforms-dropdown-item', this.$select );

		if ( value ) {
			$allItems.hide();

			$allItems.each( function( index, li ) {
				var $li = $( li) ;
				var liValue = $li.data( 'value' );
				var liLabel = ( $li.data( 'label' ).length ) ? $li.data( 'label' ).toLowerCase() : '';

				if ( liValue ) {
					liValue = liValue.toString().toLowerCase();
					value = value.toString().toLowerCase();
				}

				if ( liValue && -1 !== liValue.indexOf( value ) ) {
					$li.show();
				}

				if ( liLabel && -1 !== liLabel.indexOf( value ) ) {
					$li.show();
				}
			} );
		} else {
			$allItems.show();
		}
	}

	HappyFormsSelect.prototype.setValue = function( value ) {
		if ( 'undefined' !== typeof value ) {
			var $li = $( 'li[data-value="' + value + '"]', this.$select );

			if ( $li.length ) {
				if ( $li.data('label') ) {
					this.$el.val( $li.data( 'label' ) );
				} else {
					this.$el.val( $li.data( 'value' ) );
				}

				this.$input.val( $li.data( 'value' ) );
				this.setDefaults( $li.data( 'value' ), $li.data( 'label' ) );
			} else {
				this.$el.val( this.defaultLabel );
				this.$input.val( this.defaultValue );
			}
		}

		this.hideDropdown();
		this.$input.trigger( 'blur' );
		this.$input.trigger( 'happyforms-change' );
	}

	HappyFormsSelect.prototype.getValue = function() {
		return this.$input.val();
	}

	HappyFormsSelect.prototype.setDefaults = function( value, label ) {
		if ( value && label ) {
			this.defaultValue = value;
			this.defaultLabel = label;
		}
	}

	$.fn.happyFormsSelect = function( method ) {
        this.each(function() {
            if ( 'object' === typeof method ) {
                $.data( this, 'HappyFormSelect', new HappyFormsSelect( this, method ) );
            } else {
                var methods = $.data( this, 'HappyFormSelect' );

                if ( methods && methods[method] ) {
                    return methods[method].apply( this, Array.prototype.slice.call( arguments, 1 ) );
                }
            }
        });
    }
} )( jQuery );